#include <cstdio>
#include <cassert>

using namespace std;
struct node_t;
typedef node_t* node;
typedef long long ll;

struct node_t {
  node l, r;
  int y;
  ll first, step, count;
  ll tot_count;
  node_t(ll first, ll step, ll count);
  node_t() {
    l = r = this;
    y = 0;
    first = count = 0;
    step = 1;
    tot_count = 0;
  }
};

node_t null_node;
node null = &null_node;

int my_rand() {
  static unsigned int seed = 239017u;
  seed=seed * 1664525u + 1013904223u;
  return seed;
}

node_t::node_t(ll first, ll step, ll count) {
  this->first = first;
  this->step = step;
  this->count = this->tot_count = count;
  y = my_rand();
  l = r = null;
}


void recalc(node v) {
  v->tot_count = v->count;
  v->tot_count += v->l->tot_count;
  v->tot_count += v->r->tot_count;
}

node merge(node l, node r) {
  if (l == null) return r;
  if (r == null) return l;
  node res;
  if (l->y < r->y) {
    res = l;
    res->r = merge(res->r, r);
  } else {
    res = r;
    res->l = merge(l, res->l);
  }
  recalc(res);
  return res;
}

void split(node v, ll val, node& l, node& r) {
  if (v == null) {
    l = r = null;
    return;
  }
  if (v->first < val) {
    l = v;
    split(l->r, val, l->r, r);
    recalc(l);
    return;
  } else {
    r = v;
    split(r->l, val, l, r->l);
    recalc(r);
    return;
  }
}

node insert(node v, node u) {
  if (v == null || u->y < v->y) {
    split(v, u->first, u->l, u->r);
    recalc(u);
    return u;
  }
  if (v->first < u->first) {
    v->r = insert(v->r, u);
  } else {
    v->l = insert(v->l, u);
  }
  recalc(v);
  return v;
}

node remove(node v, ll val) {
  assert(v != null);
  if (v->first == val) {
    return merge(v->l, v->r);
  }
  if (v->first < val) {
    v->r = remove(v->r, val);
  } else {
    v->l = remove(v->l, val);
  }
  recalc(v);
  return v;
}

node get_kth(node v, ll &k) {
  if (v == null) return null;
  if (v->l->tot_count > k) {
    return get_kth(v->l, k);
  }
  k -= v->l->tot_count;
  if (v->count > k) {
    return v;
  }
  k -= v->count;
  return get_kth(v->r, k);
}

void dump_node(node v, int depth = 0) {
  if (v == null) return;
  dump_node(v->l, depth + 1);
  fprintf(stderr, "%*s%lld,%lld,%lld\n", depth, " ", v->first, v->step, v->first + v->step * (v->count - 1));
  dump_node(v->r, depth + 1);
  ll old_cnt = v->tot_count;
  recalc(v);
  assert(old_cnt == v->tot_count);
}

void split_count(node v, ll count, node& l, node& r) {
  if (v->tot_count == count) {
    l = v;
    r = null;
    return;
  }

  ll count_bak = count;
  node temp = get_kth(v, count);
  if (temp == null) {
    dump_node(v);
    fprintf(stderr, "count = %lld, v->tot_cnt = %lld\n", count_bak, v->tot_count);
  }
  assert(temp != null);
  split(v, temp->first, l, r);
  assert(count < temp->count);
  if (count != 0) {
    r = remove(r, temp->first);
    node nnode = new node_t(temp->first, temp->step, count);
    temp->first += temp->step * count;
    temp->count -= count;
    recalc(temp);
    r = insert(r, temp);
    l = insert(l, nnode);
  }
}

node do_kill(node v, int &shift) {
  if (v == null) return v;
  v->l = do_kill(v->l, shift);
  if (shift == 1) {
    v->first += v->step;
    v->count -= 1;
    shift = 0;
  }
  v->step *= 2;
  shift = v->count % 2;
  v->count = (v->count + 1) / 2;
  v->r = do_kill(v->r, shift);
  if (v->count) {
    recalc(v);
    return v;
  } else {
    return merge(v->l, v->r);
  }
}

ll get_first(node v) {
  if (v->l == null) return v->first;
  return get_first(v->l);
}

ll get_last(node v) {
  if (v->r == null) return v->first + v->step * (v->count - 1);
  return get_last(v->r); 
}


int main() {
  int n;
  scanf("%d", &n);
  ll m;
  scanf("%lld", &m);
  node root = new node_t(1, 1, (ll)(m));

  for (int i = 0; i < n; i++) {
    ll l, r;
    scanf("%lld %lld", &l, &r);
    node lf, mid, rg;
    split_count(root, r, lf, rg);
    split_count(lf, l - 1, lf, mid);
 
    printf("%lld %lld\n", get_first(mid), get_last(mid));
    int shift = 1;
    //dump_node(mid);
    mid = do_kill(mid, shift);
    //dump_node(mid);
    //printf("\n\n");
    root = merge(lf, merge(mid, rg));
  }
}
